/*
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.registry.wsclient;

import java.util.ArrayList;
import java.util.List;

import org.apache.cxf.jaxws.JaxWsProxyFactoryBean;
import org.ow2.petals.registry.api.Endpoint;
import org.ow2.petals.registry.api.Query;
import org.ow2.petals.registry.api.exception.RegistryException;
import org.ow2.petals.registry.api.ws.RemoteRegistryException;
import org.ow2.petals.registry.api.ws.service.RegistryClientService;
import org.ow2.petals.registry.client.api.RegistryClient;

/**
 * This registry client invokes the registry operations using Web Service
 * method.
 * 
 * @author Christophe HAMERLING - eBM WebSourcing
 * 
 */
public class RegistryWSClient implements RegistryClient {

    private final RegistryClientService registryClientService;

    private final EndpointAdapterImpl adapter;

    public RegistryWSClient(String wsEndpoint) {
        this.adapter = new EndpointAdapterImpl();
        JaxWsProxyFactoryBean factory = new JaxWsProxyFactoryBean();
        factory.setAddress(wsEndpoint);
        factory.setServiceClass(RegistryClientService.class);
        this.registryClientService = (RegistryClientService) factory.create();
    }
    
    public void delete(String path, boolean b) throws RegistryException {
        try {
            this.registryClientService.delete(path, b);
        } catch (RemoteRegistryException e) {
            throw new RegistryException(e.getMessage());
        }
    }

    public Endpoint get(String path, boolean b) throws RegistryException {
        Endpoint result = null;
        try {
            org.ow2.petals.registry.api.ws.to.Endpoint e = this.registryClientService.get(path, b);
            result = this.adapter.toRegistryEndpoint(e);
        } catch (RemoteRegistryException e) {
            throw new RegistryException(e.getMessage());
        }
        return result;
    }

    public boolean put(String path, Endpoint endpoint, boolean b) throws RegistryException {
        boolean result = false;
        org.ow2.petals.registry.api.ws.to.Endpoint ep = this.adapter.toWSEndpoint(endpoint);
        try {
            result = this.registryClientService.put(path, ep, b);
        } catch (RemoteRegistryException e) {
            throw new RegistryException(e.getMessage());
        }
        return result;
    }

    public List<Endpoint> getAll(String path, boolean b) throws RegistryException {
        List<Endpoint> result = null;
        try {
            List<org.ow2.petals.registry.api.ws.to.Endpoint> endpoints = this.registryClientService
                    .getAll(path, b);
            if (endpoints != null) {
                result = new ArrayList<Endpoint>();
                for (org.ow2.petals.registry.api.ws.to.Endpoint endpoint : endpoints) {
                    result.add(this.adapter.toRegistryEndpoint(endpoint));
                }
            }
        } catch (RemoteRegistryException e) {
            throw new RegistryException(e.getMessage());
        }
        return result;
    }

    public List<Endpoint> lookup(Query q, boolean b) throws RegistryException {
        // TODO Auto-generated method stub
        return null;
    }

}
